@@ -6,72 +6,72 @@
 //! [![Rust](https://img.shields.io/badge/rust-1.53%2B-blue.svg?maxAge=3600)](https://github.com/ralfbiedert/interoptopus)
 //! [![Rust](https://github.com/ralfbiedert/interoptopus/actions/workflows/rust.yml/badge.svg?branch=master)](https://github.com/ralfbiedert/interoptopus/actions/workflows/rust.yml)
 //!
-//! # Interoptopus
+//! # Interoptopus 🐙
 //!
-//! C#, Python, C, ... → 🐙 → 🦀
+//! The polyglot bindings generator for your library (C#, C, Python, ...)
 //!
-//! FFI from your favorite language to Rust. Escape hatchets included. 🪓
+//! <br>
 //!
+//! Huh?
+//! - Imagine you are writing this cool API and want the world to have it.
+//! - The world, however, is running Unity, C, Python, ... all at the same time.
+//! - "Not a problem", you say, "I'll just use Interoptopus".
 //!
-//! ## Overview
+//! And you'll live happily<sup>*</sup> ever after.
 //!
-//! If you ...
+//! <sub>*Actual results may depend on other life choices.</sub>
 //!
-//! - are **about to write** an `extern "C"` API in Rust,
-//! - **need** C#, Python, C, ... bindings to your library,
-//! - prefer having **fine-grained-control** over your API and interop generation,
-//! - would like to use **quality-of-life [patterns](crate::patterns)** on **both sides** (e.g., [options](crate::patterns::option), [slices](crate::patterns::slice), '[classes](crate::patterns::class)') where feasible,
-//! - might need to swiftly **support a new language**,
-//! - think your FFI [**single source of truth**](https://en.wikipedia.org/wiki/Single_source_of_truth) should be living Rust code,
-//!
-//! ... then Interoptopus might be for you.
-//!
-//!
-//! ## Known Limitations
-//!
-//! - not yet used in production
-//! - a bit verbose if you don't own your types (still possible, just more work)
-//! - if you target only a single language and don't care about your FFI layer other solutions might be better
-//!
-//!
-//! ## Rust Code You Write
-//!
-//! This is code you would write:
+//! ## Code you write ...
 //!
 //! ```rust
-//! use interoptopus::{ffi_function, ffi_type};
+//! use interoptopus::{ffi_function, ffi_type, inventory};
 //!
 //! #[ffi_type]
 //! #[repr(C)]
-//! pub struct Vec3 {
+//! pub struct Vec2 {
 //!     pub x: f32,
 //!     pub y: f32,
-//!     pub z: f32,
 //! }
 //!
 //! #[ffi_function]
 //! #[no_mangle]
-//! pub extern "C" fn my_function(input: Vec3) -> Vec3 {
-//!     Vec3 { x: 2.0, y: 4.0, z: input.z }
+//! pub extern "C" fn my_function(input: Vec2) {
+//!     println!("{}", input.x);
 //! }
 //!
-//! interoptopus::inventory_function!(ffi_inventory, [], [my_function], []);
+//! inventory!(ffi_inventory, [], [my_function], []);
+//!
 //! ```
 //!
-//! ## Generated Code
-//!
-//! Once you've written the code above you use one of these backends to generate interop code:
+//! ## ... Interoptopus generates
 //!
 //! | Language | Crate | Sample Output |
 //! | --- | --- | --- |
 //! | C# (incl. Unity) | [**interoptopus_backend_csharp**](https://crates.io/crates/interoptopus_backend_csharp) | [Interop.cs](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_csharp/tests/output/Interop.cs) |
 //! | C | [**interoptopus_backend_c**](https://crates.io/crates/interoptopus_backend_c) | [my_header.h](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_c/tests/output/my_header.h) |
 //! | Python [CFFI](https://cffi.readthedocs.io/en/latest/index.html) | [**interoptopus_backend_cpython_cffi**](https://crates.io/crates/interoptopus_backend_cpython_cffi) | [reference.py](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_cpython_cffi/tests/output/reference_project.py) |
-//! | Your language | Write your own backend! | - |
+//! | Your language | Write your own backend<sup>1</sup> | - |
+//!
+//! <sup>1</sup> Create your own backend in just a few hours. No pull request needed. [Pinkie promise](https://github.com/ralfbiedert/interoptopus/blob/master/FAQ.md#new-backends).
+//!
+//! ## Getting Started 🍼
+//!
+//! If you ...
+//! - want to **create a new API** see the [**example projects**](https://github.com/ralfbiedert/interoptopus/tree/master/examples),
+//! - need to **support a new language** or rewrite a backend, [**copy and adapt the C backend**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_backend_c).
 //!
 //! ## Features
 //!
-//! See the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src) lists all supported constructs including:
+//! - explicit, type-safe, **single source of truth** API definition in Rust,
+//! - **quality-of-life [patterns](crate::patterns)** on **both sides** (e.g., [options](crate::patterns::option), [slices](crate::patterns::slice), [services](crate::patterns::service), ...)
+//! - **minimal on dependencies**, build time, tooling impact,
+//! - if your **project compiles your bindings should work**<sup>&#42;*cough*&#42;</sup> (i.e., generated and callable),
+//! - **extensible**, multiple backends, **easy to support new languages**, fully **customizable**,
+//! - **no scripts needed**, `cargo build` + `cargo test` **can produce and test** (if lang installed) generated bindings
+//!
+//!
+//! ## Supported Rust Constructs
+//! See the [**reference project**](https://github.com/ralfbiedert/interoptopus/tree/master/interoptopus_reference_project/src); it lists all supported constructs including:
 //! - [functions](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/functions.rs) (`extern "C"` functions and delegates)
 //! - [types](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/types.rs) (primitives, composite, enums (numeric only), opaques, references, pointers, ...)
 //! - [constants](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_reference_project/src/constants.rs) (primitive constants; results of const evaluation)
@@ -81,6 +81,14 @@
 //! well and it's is easy to accidentally pass an outlived pointer or doubly alias a `&mut X` on reentrant functions.
 //!
 //!
+//! ## Performance 🏁
+//!
+//! Generated low-level bindings should be "zero cost" w.r.t. hand-crafted bindings for that language. However, even hand-crafted bindings
+//! have an inherent, language-specific cost. For C# that cost can be almost 0, for Python CFFI it can be high. Patterns and convenience
+//! helpers might add additional overhead.
+//!
+//! If you **need API design guidance** the following (wip) [**C# call-cost table**](https://github.com/ralfbiedert/interoptopus/blob/master/interoptopus_backend_csharp/benches/BENCHMARK_RESULTS.md)<sup>🔥</sup> can help.
+//!
 //! ## Current Status
 //!
 //! - June 20, 2021 - Alpha. Has generated simple working<sup>TM</sup> bindings for a few projects for a week now, many things missing.
@@ -181,7 +189,7 @@ pub mod lang {
 /// #[no_mangle]
 /// pub extern "C" fn f(_x: u8) {}
 ///
-/// interoptopus::inventory_function!(
+/// interoptopus::inventory!(
 ///     my_inventory_function,
 ///     [ MY_CONSTANT ],
 ///     [ f ],
@@ -195,7 +203,7 @@ pub mod lang {
 #[cfg(feature = "derive")]
 #[cfg_attr(docsrs, doc(cfg(feature = "derive")))] // does this work?
 #[macro_export]
-macro_rules! inventory_function {
+macro_rules! inventory {
     (
         $export_function:ident,
         [
@@ -241,7 +249,7 @@ macro_rules! inventory_function {
                     let pattern: $crate::patterns::LibraryPattern = $pattern().into();
 
                     match &pattern {
-                        $crate::patterns::LibraryPattern::Class(class) => {
+                        $crate::patterns::LibraryPattern::Service(class) => {
                             functions.push(class.constructor().clone());
                             functions.push(class.destructor().clone());
                             functions.extend(class.methods().iter().cloned());
